<?php

namespace FireDrum;

if ( !defined( 'ABSPATH' ) ) {
	die();
}

/**
 * Class Email_Marketing_Woocommerce_Cart
 */
class Email_Marketing_Woocommerce_Cart {
	private $plugin;

	private static $instance = null;

	public static function instantiate() {
		if ( self::$instance === null ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	private function __construct() {
		$this->plugin = Email_Marketing_Plugin::instantiate();

		add_action ( 'init', array( $this, 'on_init' ) );
		add_action ( 'wp', array( $this, 'on_wp' ) );

		// Cart link redirects
		add_action( 'template_redirect', array( $this, 'handle_redirect_to_cart' ) );
		add_filter( 'woocommerce_login_redirect', array( $this, 'override_login_redirect' ), 10, 2 );

		// Cart actions
		add_action( 'woocommerce_cart_updated', array( $this, 'cart_updated' ) );
		add_action( 'woocommerce_cart_emptied', array( $this, 'cart_emptied' ) );
		add_action( 'woocommerce_order_status_processing', array( $this, 'empty_cart_after_checkout' ) );
		add_action( 'woocommerce_order_status_completed', array( $this, 'empty_cart_after_checkout' ) );
		add_action( 'woocommerce_checkout_order_processed', array( $this, 'empty_cart_after_checkout' ) );
	}

	public function on_init() {
		add_filter( 'cron_schedules', array( $this, 'add_cron_intervals' ) );
		if ( !wp_next_scheduled( Email_Marketing_Plugin::get_branding() . '_wc_send_abandoned_cart_emails_hook' ) ) {
			wp_schedule_event( time(), Email_Marketing_Plugin::get_branding() . '_one_second', Email_Marketing_Plugin::get_branding() . '_wc_send_abandoned_cart_emails_hook' );
		}
		add_action( Email_Marketing_Plugin::get_branding() . '_wc_send_abandoned_cart_emails_hook', array( $this, 'send_abandoned_cart_emails_exec' ) );
	}

	public function on_wp() {
		$this->init_guest_cart();
	}

	public function add_cron_intervals( $schedules ) {
		$schedules[Email_Marketing_Plugin::get_branding() . '_one_second'] = array(
			'interval' => 1,
			'display'  => esc_html__( 'Every Second', Email_Marketing_Plugin::get_textdomain() )
		);

		return $schedules;
	}

	public function send_abandoned_cart_emails_exec() {
		$campaign = get_option( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_campaign' );
		$lists = $this->get_abandoned_cart_lists();

		if ( get_option( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_enabled', '1' ) === '1' && $campaign !== '' && !empty( $lists ) ) {
			$abandonment_value = get_option( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_threshold_value', '1' );
			$abandonment_unit = get_option( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_threshold_unit', 'hours' );

			$time = 0;
			switch ( $abandonment_unit ) {
				case 'hours':
					$time = $abandonment_value * 3600;
					break;
				case 'days':
					$time = $abandonment_value * 86400;
					break;
				default:
					break;
			}


			$wpdb = $this->plugin->get_wpdb();

			$abandoned_carts = $wpdb->get_results( $wpdb->prepare( "
				SELECT `id`, `user_id`, `user_email`
				FROM `{$wpdb->prefix}" . Email_Marketing_Plugin::get_branding() . '_wc_customer_carts`
				WHERE `cart_empty` = %d AND `sent_abandoned_cart_email` = %d AND `date_updated` <= DATE_SUB(NOW(), INTERVAL %d SECOND)',
				array (
					false,
					false,
					$time
				)
			), ARRAY_A );

			$api = $this->plugin->get_api();
			foreach ( $abandoned_carts as $abandoned_cart ) {
				$email = $this->get_user_email( $abandoned_cart['user_id'], $abandoned_cart['user_email'] );

				$api->add_contact_to_lists( $email, $lists );

				// Update first to err on the side of caution; it is better to not send the email at all than to send it more than once!
				$wpdb->update( $wpdb->prefix . Email_Marketing_Plugin::get_branding() . '_wc_customer_carts', array( 'sent_abandoned_cart_email' => true ), array( 'id' => $abandoned_cart['id'] ) );

				$api->send_campaign_to_contact( $campaign, $email );
			}
		}
	}

	public function get_user_email( $user_id, $user_email ) {
		if ( $user_id ) {
			$user_email = get_user_meta( $user_id, 'billing_email', true );
			if ( empty( $user_email ) ) {
				$user_email = ( new \WP_User( $user_id ) )->user_email;
			}
		}

		return $user_email;
	}

	public function get_abandoned_cart_lists() {
		$result = array();

		$api = $this->plugin->get_api();
		$categories = $api->get_categories_from_api();
		if ( isset( $categories['categories'] ) && is_array( $categories['categories'] ) ) {
			$category_id = 0;
			foreach( $categories['categories'] as $category ) {
				if ( $category['name'] === 'WooCommerce Abandoned Carts' ) {
					$category_id = $category['id'];
					break;
				}
			}

			if ( $category_id === 0 ) {
				$response = $api->add_category( 'WooCommerce Abandoned Carts' );
				if ( isset( $response['category'] ) && isset( $response['category']['id'] ) && $response['category']['id'] !== 0 ) {
					delete_transient( Email_Marketing_Plugin::get_branding() . '_categories_response' );
					$categories = $api->get_categories_from_api();
					$result[] = $response['category']['id'];
				}
			} else {
				$result[] = $category_id;
			}
		}

		return $result;
	}

	/**
	 * Load the guest cart if customer is not logged in and attempting
	 * to view the cart from a link generated by a cart email
	 */
	public function init_guest_cart() {
		if ( is_user_logged_in() || !is_cart() ) {
			return;
		}

		if ( !isset( $_GET['cartid'] ) ) {
			return;
		}

		$wpdb = $this->plugin->get_wpdb();
		$carts = $wpdb->get_results( $wpdb->prepare( "SELECT `user_id`, `user_email` FROM {$wpdb->prefix}" . Email_Marketing_Plugin::get_branding() . '_wc_customer_carts WHERE `id` = %d', $_GET['cartid'] ), ARRAY_A );

		if ( empty( $carts ) ) {
			return;
		}

		if ( $carts[0]['user_id'] === 0 ) {
			WC()->cart = null;
			$this->init_wc_cart( 0, $carts[0]['user_email'] );

			wp_redirect( get_permalink( woocommerce_get_page_id( 'cart' ) ) );
			exit;
		}
	}

	/**
	 * Handle [cart_url] links by making sure the customer is logged in first
	 * before viewing the cart to allow WC to load his persistent cart.
	 */
	public function handle_redirect_to_cart() {
		if ( is_user_logged_in() || !is_cart() ) {
			return;
		}

		if ( !isset( $_GET['cartid'] ) ) {
			return;
		}

		$wpdb = $this->plugin->get_wpdb();
		$carts = $wpdb->get_results( $wpdb->prepare( "SELECT `user_id`, `user_email` FROM {$wpdb->prefix}" . Email_Marketing_Plugin::get_branding() . '_wc_customer_carts WHERE `id` = %d', $_GET['cartid'] ), ARRAY_A );

		if ( empty( $carts ) ) {
			return;
		}

		if ( $carts[0]['user_id'] > 0 ) {
			// set a cookie to force a redirect to the cart after logging in
			setcookie( Email_Marketing_Plugin::get_branding() . '_cart_redirect', true, 0, SITECOOKIEPATH, COOKIE_DOMAIN );
			wc_add_notice( __( 'Please log in to view your saved cart.', Email_Marketing_Plugin::get_textdomain() ), 'notice' );
			wp_redirect( get_permalink( wc_get_page_id( 'myaccount' ) ) );
			exit;
		}
	}

	/**
	 * Override the redirect URL to point to the cart if a '<branding>_cart_redirect' cookie is present.
	 *
	 * @param string $redirect
	 * @param WP_User $user
	 *
	 * @return string
	 */
	public function override_login_redirect($redirect, $user) {
		if ( isset( $_COOKIE[Email_Marketing_Plugin::get_branding() . '_cart_redirect'] ) ) {
			$redirect = WC()->cart->get_cart_url();
		}

		return $redirect;
	}

	/**
	 * Runs when the cart is updated. Load the persistent cart and
	 * look for items in the cart that needs to be added to the queue
	 */
	public function cart_updated() {
		// Only if user is logged in or an email is stored in the session
		$user = wp_get_current_user();
		$email = self::get_session_email();

		if ( 0 === $user->ID && !$email ) {
			return;
		}

		$cart = WC()->cart->get_cart();

		if ( empty( $cart ) ) {
			// Cart has been emptied. we need to remove existing email orders for this user
			$this->cart_emptied();
		} else {
			self::clone_cart();
		}
	}

	/**
	 * When the cart is emptied, clear all queued unsent cart emails
	 * @param int $user_id
	 */
	public function cart_emptied($user_id = 0) {
		if ( !$user_id ) {
			$user_id = get_current_user_id();
		}

		do_action( "{$this->plugin::get_branding()}_wc_cart_emptied" );

		self::clone_cart();
	}

	/**
	 * If the order is from a registered customer, clear all cart emails
	 * that have been created for the customer.
	 *
	 * @param int $order_id
	 */
	public function empty_cart_after_checkout($order_id) {
		self::clone_cart( get_post_meta( $order_id, '_customer_user', true ), get_post_meta( $order_id, '_billing_email', true ), true );
		
	}

	/**
	 * Load the cart for the current user and store a duplicate
	 * in the <branding>_wc_customer_carts table
	 *
	 * @param int $user_id Leave empty to default to the current user
	 */
	public static function clone_cart($user_id = 0, $user_email = '', $reset_sent_abandoned_cart_email_status = false) {
		$wpdb = self::$instance->plugin->get_wpdb();

		if ( !$user_id ) {
			$user_id = get_current_user_id();
		}

		if ( $user_email === '' ) {
			$user_email = WC()->session->get( 'wc_guest_email', '' );
		}
		//$name = WC()->session->get( 'wc_guest_name', array( '', '' ) );
                  
		$wc_cart = WC()->cart;
                if( empty($wc_cart)) {
                  return;
                }

		if ( !$user_id && !$user_email ) {
			return;
		}

		if ( $user_email ) {
			$cart_id = $wpdb->get_var( $wpdb->prepare( "SELECT id FROM {$wpdb->prefix}" . Email_Marketing_Plugin::get_branding() . '_wc_customer_carts WHERE user_email = %s', $user_email ) );
		} else {
			$cart_id = $wpdb->get_var( $wpdb->prepare( "SELECT id FROM {$wpdb->prefix}" . Email_Marketing_Plugin::get_branding() . '_wc_customer_carts WHERE user_id = %d', $user_id ) );
		}

		$cart = $wc_cart->get_cart();
		$cart_data = array(
			'user_id' => $user_id,
			'first_name' => !empty($name[0]) ?: '',
			'last_name' => !empty($name[1]) ?: '',
			'user_email' => $user_email,
			'cart_empty' => empty( $cart ),
			'cart_items' => serialize( $cart ),
			'cart_total' => $wc_cart->cart_contents_total,
			'date_updated' => current_time( 'mysql' )
		);

		if ( $reset_sent_abandoned_cart_email_status ) {
			$cart_data['sent_abandoned_cart_email'] = false;
		}

		if ( !$cart_id ) {
			$wpdb->insert( $wpdb->prefix . Email_Marketing_Plugin::get_branding() . '_wc_customer_carts', $cart_data );
		} else {
			$wpdb->update( $wpdb->prefix . Email_Marketing_Plugin::get_branding() . '_wc_customer_carts', $cart_data, array( 'id' => $cart_id ) );
		}
	}

	/**
	 * Return the contents of a customer's cart
	 *
	 * @param int $user_id
	 * @param string $email
	 * @return array|false
	 */
	public static function get_cart($user_id = 0, $email = '') {
		$wpdb = self::$instance->plugin->get_wpdb();

		if ( empty( $user_id ) && empty( $email ) ) {
			$user_id = get_current_user_id();
			$email   = self::get_session_email();
		}

		if ( $user_id ) {
			$cart_row = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}" . Email_Marketing_Plugin::get_branding() . '_wc_customer_carts WHERE user_id = %d', $user_id ), ARRAY_A );
		} else {
			$cart_row = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM {$wpdb->prefix}" . Email_Marketing_Plugin::get_branding() . '_wc_customer_carts WHERE user_email = %s', $email ), ARRAY_A );
		}

		if ( !$cart_row ) {
			$cart_row = false;
		} else {
			$cart_row['cart_items'] = maybe_unserialize( $cart_row['cart_items'] );
		}

		return $cart_row;
	}

	/**
	 * Initialize the WC_Cart class so it can be used in the admin panel
	 * @param int       $user_id
	 * @param string    $email
	 */
	public static function init_wc_cart($user_id = 0, $email = '') {
		if ( isset( WC()->cart ) && is_a( WC()->cart, 'WC_Cart' ) ) {
			return;
		}

		Email_Marketing_Woocommerce::init_wc_session();

		if ( !function_exists( 'wc_cart_totals_order_total_html' ) ) {
			include_once( WC()->plugin_path() . '/includes/wc-cart-functions.php' );
		}

		$cart_row = self::get_cart( $user_id, $email );

		WC()->cart = new WC_Cart();

		if ( $cart_row ) {
			WC()->session->cart = $cart_row['cart_items'];
		}

		WC()->customer = new WC_Customer();
		WC()->cart->init();
	}

	/**
	 * Return the current user's session email if it exists
	 * @return string
	 */
	public static function get_session_email() {
		if ( !WC()->session ) {
			Email_Marketing_Woocommerce::init_wc_session();
		}

		return WC()->session->get( 'wc_guest_email', '' );
	}
}
